package org.netkernel.layer1.endpoint;

import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.urii.FileExpiry;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.w3c.dom.*;
import java.util.*;
import java.io.File;
import java.net.URI;
import java.util.regex.*;

public class FLSAccessor extends StandardAccessorImpl
{
	public static final int LS_RECURSIVE=1;
	public static final int LS_SHOW_URI=4;
	public static final int LS_SHOW_MODULE=8;
	public static final int LS_SHOW_LENGTH=16;
	public static final int LS_SHOW_LAST_MODIFIED=32;
	public static final int LS_SHOW_AGE=64;
	public static final int LS_TRIM_EMPTY=128;
	
	
	private static final String NN_DIRECTORY="dir";
	private static final String NN_RESOURCE="res";	
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		IHDSNode operator = aContext.source("arg:operator",IHDSNode.class);
		String root;
		File f=null;
		int flag = 0;
		if (operator.getFirstNode("/fls/recursive")!=null) flag|=LS_RECURSIVE;
		if (operator.getFirstNode("/fls/keepEmptyDir")==null) flag|=LS_TRIM_EMPTY;
		if (operator.getFirstNode("/fls/uri")!=null) flag|=LS_SHOW_URI;
		if (operator.getFirstNode("/fls/root")==null) throw new Exception("Must declare root directory in element <root>");
		else
		{	root=operator.getFirstValue("/fls/root").toString();
			f=new File(URI.create(root));
			if(!f.isDirectory()) throw new Exception("Root must be directory");
		}
					
 		if (operator.getFirstNode("/fls/length")!=null) flag|=LS_SHOW_LENGTH;
		if (operator.getFirstNode("/fls/lastmodified")!=null) flag|=LS_SHOW_LAST_MODIFIED;
		if (operator.getFirstNode("/fls/age")!=null) flag|=LS_SHOW_AGE;
		Matcher filter = null;
		if (operator.getFirstValue("/fls/filter")!=null)
		{	String filterRegex = operator.getFirstValue("/fls/filter").toString();
			filter = Pattern.compile(filterRegex).matcher("");
		}
		
		Document d = listResources(root,f,flag,filter);
		INKFResponse resp=aContext.createResponseFrom(d);
		resp.setExpiry(INKFResponse.EXPIRY_FUNCTION, new FileExpiry(f, getKernel().getConfiguration().getLong("netkernel.poll",1000L)));
	}
	
	
	public Document listResources(String aRoot, File f,int aFlags, Matcher aFilter) throws Exception
	{	Document d = XMLUtils.newDocument();
		Element root = d.createElement("fls");
		d.appendChild(root);
		StringBuffer path = new StringBuffer(256);
		path.append(aRoot);
		recurseDir(f,root,path,false, aFlags, aFilter);
		
		return d;
	}
	
	private int recurseDir(File aFile, Node aNode, StringBuffer aPath, boolean aExternal, int aFlags, Matcher aFilter) throws Exception
	{	int count=0;
		long time = System.currentTimeMillis();
		File[] files = aFile.listFiles();
		if (files == null)
		{	return 0;
		}
		int pathLength = aPath.length();
		for (int i=0; i<files.length; i++)
		{	File f = files[i];
			String name = f.getName();
			boolean isDirectory = f.isDirectory();
			aPath.setLength(pathLength);
			if (isDirectory)
			{	aPath.append(name);
				aPath.append('/');
				//String path = aPath+name+'/';
				Node child = ensureChild(aNode,name,isDirectory,false);
				int recurseCount=0;
				if((aFlags&LS_RECURSIVE)>0) recurseCount=recurseDir(f,child,aPath,aExternal,aFlags,aFilter);
				if (recurseCount>0)
				{	count+=recurseCount;
				}
				else if ((aFlags&LS_TRIM_EMPTY)>0)
				{	if (XMLUtils.getFirstChildElement(child)==null)
					{	aNode.removeChild(child);
					}
				}
			}
			else
			{	aPath.append(name);
				//String path = aPathaBuffer.toString();
				boolean unfiltered = true;
				if (aFilter!=null)
				{	aFilter.reset(aPath);
					unfiltered = aFilter.matches();
				}
				if (unfiltered)
				{	if (!aExternal)
					{	Node child = ensureChild(aNode,name,isDirectory,true);
						if (child!=null)
						{	count++;
							if ((aFlags&LS_SHOW_URI)>0)
								XMLUtils.appendTextedElement(child, "uri", f.toURI().toString().trim());
							if ((aFlags&LS_SHOW_LENGTH)>0)
								XMLUtils.appendTextedElement(child, "length", Long.toString(f.length()));
							if ((aFlags&LS_SHOW_LAST_MODIFIED)>0)
								XMLUtils.appendTextedElement(child, "lastmodified", Long.toString(f.lastModified()));
							if ((aFlags&LS_SHOW_AGE)>0)
								XMLUtils.appendTextedElement(child, "age", Long.toString(time-f.lastModified()));
						}
					}
				}
			}
		}
		return count;
	}
	
	private String escape(String aString)
	{	return aString.replaceAll(" ", "%20");
	}
	
	private static Node ensureChild(Node aParent, String aName, boolean aIsDirectory, boolean aMustCreate) throws Exception
	{	Node result = null;
		String name = aIsDirectory?NN_DIRECTORY:NN_RESOURCE;
		for (Node n = aParent.getFirstChild(); n!=null; n=n.getNextSibling())
		{	if (n.getNodeName().equals(name))
			{	String childName = XMLUtils.getText(n);
				if (childName.equals(aName))
				{	result = n;
					break;
				}
			}
		}
		if (result==null)
		{	Element e = aParent.getOwnerDocument().createElement(name);
			XMLUtils.setText(e, aName);
			aParent.appendChild(e);
			result = e;
		}
		else if (aMustCreate)
		{	result = null;
		}
		
		return result;
	}
	
	private Node ensurePath(String aPath, Document aListingDoc) throws Exception
	{	StringTokenizer st=new StringTokenizer(aPath,"/");
		Node n = aListingDoc.getDocumentElement();
		while (st.hasMoreTokens())
		{	String childName = st.nextToken();
			boolean isDir = st.hasMoreTokens();
			n = ensureChild(n, childName, isDir, !isDir);
		}
		return n;
	}
	
}
