package org.netkernel.layer1.endpoint;

import java.util.*;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.urii.ParsedIdentifierImpl;
import org.netkernel.layer0.util.RequestBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.w3c.dom.Node;

public class ForEachAccessor extends StandardAccessorImpl
{	
	public ForEachAccessor()
	{	this.declareThreadSafe();		
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		//determine mode
		String activeType=aContext.getThisRequest().getArgumentValue(ParsedIdentifierImpl.ARG_ACTIVE_TYPE);
		boolean async=activeType.contains("Async");
		Long timeout=Long.MAX_VALUE;
		if(async && aContext.getThisRequest().argumentExists("timeout"))
		{	timeout=aContext.source("arg:timeout", Long.class);
		}
		boolean tolerant=activeType.contains("Tolerant");
		
		//get operand list
		Iterable operand=aContext.source("arg:operand",Iterable.class);
		List operandList = new ArrayList();
		for (Object item: operand)
		{	operandList.add(item);
		}
		
		//determine do request
		Node req=aContext.source("arg:do",Node.class);
		RequestBuilder doRequestBuilder=new RequestBuilder(req, aContext.getKernelContext().getKernel().getLogger());
		
		//issue the requests
		int count=operandList.size();
		List responseList = new ArrayList(operandList.size());
		for (int index=0; index<count; index++)
		{
			Object item=operandList.get(index);
			RequestBuilder.Arguments args = new RequestBuilder.Arguments();
			args.addArgumentByValue("arg:item", item);
			args.addArgumentByValue("arg:count", count);
			args.addArgumentByValue("arg:index", index);
			INKFRequest doReq=doRequestBuilder.buildRequest(aContext, args, null);
			if (async)
			{	INKFAsyncRequestHandle handle=aContext.issueAsyncRequest(doReq);
				responseList.add(handle);
			}
			else
			{	Object respIn;
				try
				{	respIn=aContext.issueRequestForResponse(doReq);
				}
				catch (NKFException e)
				{	if (tolerant)
					{	respIn=e;
					}
					else
					{	throw e;
					}
				}
				responseList.add(respIn);
			}
		}
		
		//join async requests if necessary
		if (async)
		{	for (int i=0; i<responseList.size(); i++)
			{	INKFAsyncRequestHandle handle=(INKFAsyncRequestHandle)responseList.get(i);
				Object respIn;
				try
				{	respIn=handle.joinForResponse(timeout);
					if(respIn==null)
					{	//Timed Out
						if(!tolerant)
						{	throw new NKFException("forEach Timeout", "For operand item number "+i+" timed out after "+timeout);							
						}
					}
				}
				catch (NKFException e)
				{	if (tolerant)
					{	respIn=e;
					}
					else
					{	throw e;
					}
				}
				responseList.set(i,respIn);
			}
		}
		
		//build merge request
		RequestBuilder mergeRequestBuilder=null;
		if (aContext.getThisRequest().argumentExists("merge"))
		{	req=aContext.source("arg:merge",Node.class);
			mergeRequestBuilder=new RequestBuilder(req, aContext.getKernelContext().getKernel().getLogger());
		}
		
		//Create Representation List if needed
		ArrayList representationList=new ArrayList(responseList.size());
		for(Object resp: responseList)
		{	Object rep;
			if(resp instanceof INKFResponseReadOnly)
			{	INKFResponseReadOnly r=(INKFResponseReadOnly)resp;
				rep=r.getRepresentation();
			}
			else
			{	rep=resp;	//Must be exception or null from timeout
			}
			representationList.add(rep);
		}
		
		//merge responses/representations or return response collection
		if (mergeRequestBuilder!=null)
		{	RequestBuilder.Arguments args = new RequestBuilder.Arguments();
			args.addArgumentByValue("arg:responses", Collections.unmodifiableList(responseList));
			args.addArgumentByValue("arg:representations", Collections.unmodifiableList(representationList));
			INKFRequest mergeReq=mergeRequestBuilder.buildRequest(aContext, args, null);
			INKFResponseReadOnly respIn=aContext.issueRequestForResponse(mergeReq);
			aContext.createResponseFrom(respIn);
		}
		else
		{	Object result;
			if(async && timeout==0L)
			{	result=null;		//Fire and Forget Mode				
			}
			else
			{	result=Collections.unmodifiableList(representationList);
			}
			aContext.createResponseFrom(result);
		}
	}
}