package org.netkernel.layer1.endpoint;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.representation.impl.HDSFactory;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;
import org.w3c.dom.Document;

/**
 * XML to HDS transreptor
 */
public class HDSToXML extends StandardTransreptorImpl
{
	
	public HDSToXML()
	{	this.declareThreadSafe();
		this.declareFromRepresentation(IHDSNode.class);
		this.declareFromRepresentation(IHDSNodeList.class);
		this.declareToRepresentation( Document.class);
		this.declareToRepresentation( IReadableBinaryStreamRepresentation.class);
		this.declareName("HDSToXML");
		this.declareDescription("Transrepts from HDS to XML");
	}
	
	public void onTransrept(INKFRequestContext aContext) throws Exception
	{	INKFResponseReadOnly primaryResponse = aContext.getThisRequest().getPrimaryAsResponse();
		Object primary=primaryResponse.getRepresentation();
		Class repClass=aContext.getThisRequest().getRepresentationClass();
		boolean wantDOM=repClass.isAssignableFrom(Document.class);
		Document d;
		
		if (primary instanceof IHDSNode)
		{	IHDSNode n=(IHDSNode)primary;
			d=HDSFactory.toDOM(n,wantDOM);
		}
		else
		{	IHDSNodeList nl=(IHDSNodeList)primary;
			d=HDSFactory.toDOM(nl,wantDOM);
		}
		INKFResponse response;
		if (wantDOM)
		{	response=aContext.createResponseFrom(d);
		}
		else
		{	IReadableBinaryStreamRepresentation rep=XMLUtils.toXML(d, false, false, "UTF-8");
			response=aContext.createResponseFrom(rep);
		}
		if(primaryResponse.getMimeType().equals(INKFResponseReadOnly.MIME_UNKNOWN))
		{	response.setMimeType("application/xml");
		}
	}
}