/******************************************************************************
  (c) Copyright 2002-2006, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  File:          $RCSfile$
  Version:       $Name$ $Revision$
  Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer1.endpoint;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.tools.ExtraMimeTypes;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.layer1.representation.DeterminateStringRep;
import org.netkernel.layer1.representation.IDeterminateStringRepresentation;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;
import org.w3c.dom.*;

public class ToDeterminateString extends StandardTransreptorImpl
{
	public ToDeterminateString()
	{	this.declareThreadSafe();
		this.declareToRepresentation(IDeterminateStringRepresentation.class);
	}
	
	public void onTransrept(INKFRequestContext aHelper) throws Exception
	{	
		INKFResponseReadOnly primaryResponse = aHelper.getThisRequest().getPrimaryAsResponse();
		Object primaryRep = primaryResponse.getRepresentation();
		String result;
		if (primaryRep instanceof String)
		{	result = (String)primaryRep;
		}
		else if (primaryRep instanceof IHDSNode)
		{	IHDSNode n = (IHDSNode)primaryRep;
			if (n.getRoot()==n)
			{	IHDSNode[] children=n.getChildren();
				if (children.length==1)
				{	n=children[0];
				}
				else
				{	throw aHelper.createFormattedException("EX_TOCAUSALSTRING_BAD_HDS_NODE", null, null,null);
				}
			}
			Object value = n.getValue();
			if (value==null)
			{	throw aHelper.createFormattedException("EX_TOCAUSALSTRING_BAD_HDS_NULL", null, null,null);
			}
			else
			{	result = n.getValue().toString();
			}
		}
		else if (primaryRep instanceof Node)
		{	result = processDOM((Node)primaryRep, aHelper);
		}
		else if (primaryRep instanceof IBinaryStreamRepresentation)
		{
			String mime=primaryResponse.getMimeType();
			if (ExtraMimeTypes.getInstance().isAssignableAFromB("text/xml", mime))
			{	Document d = aHelper.sourcePrimary(Document.class);
				result = processDOM(d,aHelper);
			}
			else
			{	result = aHelper.sourcePrimary(String.class);
			}
		}
		else
		{	throw aHelper.createFormattedException("EX_TOCAUSALSTRING_UNSUPPORTED_PRIMARY", primaryRep.getClass().getName(), null, null);
		}
		
		aHelper.createResponseFrom(new DeterminateStringRep(result));
	}
	
	private String processDOM(Node aNode, INKFRequestContext aHelper) throws NKFException
	{	String result;
		switch (aNode.getNodeType())
		{	case Node.DOCUMENT_NODE:
			{	Element docEl = ((Document)aNode).getDocumentElement();
				if (docEl==null)
				{	throw aHelper.createFormattedException("EX_TOCAUSALSTRING_BAD_DOM", null, null,null);
				}
				else
				{	result = XMLUtils.getText(docEl);
				}
				break;
			}
			case Node.ELEMENT_NODE:
				result = XMLUtils.getText(aNode);
				break;
			case Node.ATTRIBUTE_NODE:
				result = aNode.getNodeValue();
				break;
			default:
				throw aHelper.createFormattedException("EX_TOCAUSALSTRING_BAD_DOM", null, null,null);
		}
		return result;
	}
		
	
}