/******************************************************************************
 * (c) Copyright 2002,2003, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile: ExecProcessAccessor.java,v $
 * Version:       $Name:  $ $Revision: 1.6 $
 * Last Modified: $Date: 2005/05/19 10:08:49 $
 *****************************************************************************/
package org.netkernel.layer1.endpoint.exec;

import org.netkernel.layer0.meta.IEndpointArgumentMeta;
import org.netkernel.layer0.meta.impl.EndpointArgumentMetaImpl;
import org.netkernel.layer0.meta.impl.EndpointMetaImpl;
import org.netkernel.layer0.meta.impl.IdentifierArgumentMetaImpl;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.urii.ParsedIdentifierImpl;
import org.netkernel.layer1.representation.IDeterminateStringArray;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

import java.io.*;
import java.net.URI;
import java.util.*;

/**
 * Wrap an external exec-ed process
 * @author  tab
 */
public class ExecProcessAccessor extends StandardAccessorImpl
{
	private static final String ARG_COMMAND="arg:command";
	private static final String ARG_STDIN="arg:stdin";
	private static final String ARG_PARAM="arg:param";
	private static final String ARG_STDERR="stderr";
	private static final String ARG_WD="wd";
	
	/** Creates a new instance of AsyncJoinAccessor */
	public ExecProcessAccessor()                                                                  
	{	declareThreadSafe();
		IEndpointArgumentMeta m=new EndpointArgumentMetaImpl("command", null, null, null, null, true, new Class[]{IDeterminateStringArray.class});
		declareArgument(m);
		m=new EndpointArgumentMetaImpl("stdin", null, null, null, null, true, new Class[]{IBinaryStreamRepresentation.class});
		declareArgument(m);
		declareArgument(new IdentifierArgumentMetaImpl("stderr", null, "active:exec-stderr"));
		//m=new EndpointArgumentMetaImpl("stderr", null, null, null, "active:exec-stderr", false, null);
		//declareArgument(m);
		m=new EndpointArgumentMetaImpl("param", null, null, null, null, true, new Class[]{IHDSNode.class});
		declareArgument(m);
		//m=new EndpointArgumentMetaImpl("wd", null, null, null, null, false, null);
		//declareArgument(m);
		declareArgument(new IdentifierArgumentMetaImpl("wd", null, null));

		
		declareSourceRepresentation(IBinaryStreamRepresentation.class);
	}
	
	public void onSource(INKFRequestContext context) throws Exception
	{	String type = context.getThisRequest().getArgumentValue(ParsedIdentifierImpl.ARG_ACTIVE_TYPE);
		if (type.equals("exec"))
		{	exec(context);
		}
		else if (type.equals("exec-stderr"))
		{	handleStderr(context);
		}
	}	
	
	private void exec(INKFRequestContext context) throws Exception
	{	String[] command = getCommand(context);
		IBinaryStreamRepresentation stdinBinary=getStdin(context);
		boolean ignoreReturnCode=false;
		boolean useDefaultStderrHandler=true;
		String mime="text/plain";
		if (context.getThisRequest().argumentExists("param"))
		{	IHDSNode param = context.source(ARG_PARAM,IHDSNode.class);
			ignoreReturnCode= param.getFirstNode("//ignoreReturnCode")!=null;
			useDefaultStderrHandler= param.getFirstNode("//ignoreStderr")==null;
			if (param.getFirstNode("//mimetype")!=null)
			{	mime=param.getFirstValue("//mimetype").toString();
			}
		}
		String stderrURI=context.getThisRequest().getArgumentValue(ARG_STDERR);
		if (stderrURI==null && useDefaultStderrHandler)
		{	stderrURI = "active:exec-stderr";
		}
		File wd = getWorkingDirectory(context);
		
		
		ExecProcessResponseAspect aspect = new ExecProcessResponseAspect(command,stdinBinary,stderrURI,wd,ignoreReturnCode,context);
		INKFResponse response = context.createResponseFrom(aspect);
		response.setMimeType(mime);
	}
	
	private String[] getCommand(INKFRequestContext context) throws Exception
	{	String[] result=context.source(ARG_COMMAND,IDeterminateStringArray.class).getStringArray();
		if (result.length==0)
		{	throw new NKFException("command argument array is zero length");
		}
		return result;
	}
	
	private IBinaryStreamRepresentation getStdin(INKFRequestContext context) throws Exception
	{	IBinaryStreamRepresentation result=null;
		if (context.getThisRequest().argumentExists("stdin"))
		{	result = (IBinaryStreamRepresentation)context.source(ARG_STDIN, IBinaryStreamRepresentation.class);
		}
		return result;
	}
	
	private File getWorkingDirectory(INKFRequestContext context) throws Exception
	{	String wdURIString=context.getThisRequest().getArgumentValue(ARG_WD);
		File wd=null;
		if (wdURIString!=null)
		{	URI wdURI = URI.create(wdURIString);
			if (!wdURI.getScheme().equals("file"))
			{	throw new NKFException("Working Directory invalid","working directory must be specified as a file: scheme URI",null);
			}
			else
			{	wd = new File(wdURI);
			}
		}
		return wd;
	}
	
	private void handleStderr(INKFRequestContext context) throws Exception
	{	
		IBinaryStreamRepresentation stderr=context.source("arg:stderr",IBinaryStreamRepresentation.class);
		ByteArrayOutputStream baos=new ByteArrayOutputStream(1024);
		stderr.write(baos);
		baos.flush();
		
		byte[] b=baos.toByteArray();
		ByteArrayInputStream bais = new ByteArrayInputStream(b);
		BufferedReader reader = new BufferedReader(new InputStreamReader(bais));
		
		//InputStream stream = ((IReadableBinaryStreamRepresentation)context.source("arg:stderr",IReadableBinaryStreamRepresentation.class)).getInputStream();
		//String command = context.source("arg:command",String.class);
		String line;
		while ((line=reader.readLine())!=null)
		{	String message = "stderr from exec: "+line;
			context.logRaw(INKFRequestContext.LEVEL_WARNING, message);
		}
	}
	
}